from __future__ import annotations

from pathlib import Path

# Project root (repo root) when running from a source checkout.
PROJECT_ROOT = Path(__file__).resolve().parents[2]

OUTPUTS_DIR = PROJECT_ROOT / "outputs"
